package com.example.afrishop_v3.controllers;

import com.example.afrishop_v3.base.Result;
import com.example.afrishop_v3.base.StateConstant;
import com.example.afrishop_v3.enums.ResultCodeEnum;
import com.example.afrishop_v3.models.TbCfCartRecordR;
import com.example.afrishop_v3.models.TbCfExpressTemplate;
import com.example.afrishop_v3.models.TbCfItemDetail;
import com.example.afrishop_v3.repository.TbCfCartRecordRRepository;
import com.example.afrishop_v3.repository.TbCfExpressTemplateRepository;
import com.example.afrishop_v3.repository.TbCfItemDetailRepository;
import com.example.afrishop_v3.security.services.AuthenticationUser;
import com.example.afrishop_v3.util.ValidateUtils;
import com.example.afrishop_v3.util.WordposHelper;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;

@RestController
@RequestMapping("/cart")
public class CartController extends Controller {
    private final TbCfCartRecordRRepository repository;
    private final TbCfItemDetailRepository itemDetailRepository;
    private final TbCfExpressTemplateRepository templateRepository;
    private final AuthenticationUser user;

    public CartController(TbCfCartRecordRRepository repository, TbCfItemDetailRepository itemDetailRepository, TbCfExpressTemplateRepository templateRepository, AuthenticationUser user) {
        this.repository = repository;
        this.itemDetailRepository = itemDetailRepository;
        this.templateRepository = templateRepository;
        this.user = user;
    }

    @PostMapping
    public Result addToCart(@RequestBody TbCfItemDetail itemDetail){

        String userId = this.user.userId();

        if( itemDetail == null ){
            return new Result(ResultCodeEnum.SERVICE_ERROR.getCode(),"Request body is empty");
        }

        int count = repository.countByUserId(userId);

        if( count > 99 ){
            return new Result(ResultCodeEnum.SERVICE_ERROR.getCode(),"Your shopping cart is full");
        }

        BigDecimal price = itemDetail.getItemPrice();

        if ( price == null || price.compareTo(BigDecimal.ZERO) <= 0) {
            return new Result(ResultCodeEnum.SERVICE_ERROR.getCode(),"The price of the goods is incorrect");
        }


        Optional<TbCfItemDetail> optionalItem;
        String itemId = itemDetail.getItemId();
        boolean hasItemId = itemId != null && !itemId.isEmpty();
        if( hasItemId ){
            optionalItem = itemDetailRepository.findFirstByItemImgAndItemSku(itemDetail.getItemImg(), itemDetail.getItemSku());
        }else {
            optionalItem = itemDetailRepository.findFirstBySourceItemIdAndItemSku(itemDetail.getSourceItemId(),itemDetail.getItemSku());
        }

        TbCfItemDetail detail;

        if ( optionalItem.isPresent() ){

            detail = optionalItem.get();
            detail.setGoodsId(itemId);

            itemDetailRepository.save(detail);
            itemId = detail.getItemId();

        }else{
            itemId = uid();

            try {
                detail = insertItem(itemDetail,itemId);
            } catch (Exception e) {
                return new Result(ResultCodeEnum.SERVICE_ERROR.getCode(),e.getMessage());
            }
        }

        // Check if the item exist in cart
        Optional<TbCfCartRecordR> cartOptional = repository.findFirstByUserIdAndItemDetailItemIdAndItemSku(userId, itemId, itemDetail.getItemSku());

        if ( cartOptional.isPresent() ){
            TbCfCartRecordR cart = cartOptional.get();
            cart.increaseNum(itemDetail.getItemNum());
            repository.save(cart);
        }else{
            detail.setItemNum(itemDetail.getItemNum());
            insertRecord(detail, userId);
        }

        return new Result();
    }


    @GetMapping
    public Result getItemCartList(){
        return new Result<>(repository.findAllByUserId(user.userId()));
    }

    @DeleteMapping("/delete")
    public Result deleteItems(@RequestBody String[] ids) {
        if (ids != null) {
            repository.deleteAllByCartRecordIdIn(ids);
            return new Result();
        }
        return new Result(ResultCodeEnum.SERVICE_ERROR.getCode(),
                ResultCodeEnum.SERVICE_ERROR.getDesc());
    }


    private void insertRecord(TbCfItemDetail itemDetail, String userId) {
        TbCfCartRecordR tbCfCartRecordREntity = new TbCfCartRecordR();
        tbCfCartRecordREntity.setCartRecordId(uid());
        tbCfCartRecordREntity.setCheckFlag(StateConstant.INVALID);
        tbCfCartRecordREntity.setEnableFlag(StateConstant.VALID);
        tbCfCartRecordREntity.setItemDetail(itemDetail);
        tbCfCartRecordREntity.setUserId(userId);
        tbCfCartRecordREntity.setCreateTime(new Date());
        tbCfCartRecordREntity.setItemNum(itemDetail.getItemNum());
        tbCfCartRecordREntity.setItemSku(itemDetail.getItemSku());
        tbCfCartRecordREntity.setSourceItemId(itemDetail.getSourceItemId());
        tbCfCartRecordREntity.setItemPrice(itemDetail.getItemPrice());
        repository.save(tbCfCartRecordREntity);
    }


    private TbCfItemDetail insertItem(TbCfItemDetail tbCfItemDetail, String itemId) throws Exception {

        tbCfItemDetail.setCreateTime(new Date());
        //为商品分类,后面计算运费
        try{

            TbCfExpressTemplate expressTemplate = recognizeItemCategory(tbCfItemDetail.getItemTitle());
            if (expressTemplate!=null)

                tbCfItemDetail.setItemCategory(expressTemplate.getTemplateId());
        }catch (Exception ignored){

        }
        //加入商品详情
        tbCfItemDetail.setGoodsId(tbCfItemDetail.getItemId());
        tbCfItemDetail.setStatus(1);
        //设置为默认一件商品（为了保证前人数据库信息完整性，无实际作用）
        tbCfItemDetail.setItemNum(1);
        if (itemId != null) {
            tbCfItemDetail.setItemId(itemId);
        } else {
            tbCfItemDetail.setItemId(tbCfItemDetail.getItemId());

        }
        return itemDetailRepository.save(tbCfItemDetail);
        //logger.info("插入一条商品数据！");
    }
    private TbCfExpressTemplate recognizeItemCategory(String itemTitle) throws ExecutionException, InterruptedException, TimeoutException {
        Map<String, Object> wordResult = separateText(itemTitle);
        //pos_code 16为名词，用名词去匹配; 23非汉字串
        JSONArray baseTokens = (JSONArray) wordResult.get("base_tokens");
        List<String> keywordList = new ArrayList<>();
        for (Object baseToken : baseTokens) {
            JSONObject jsonObject = (JSONObject) baseToken;
            if (16 == (Integer) jsonObject.get("pos_code")) {
                keywordList.add((String) jsonObject.get("word"));
            }
            if (23 == (Integer) jsonObject.get("pos_code")) {
                keywordList.add((String) jsonObject.get("word"));
            }
        }
        //总的可能适用的运费模板
        Set<TbCfExpressTemplate> totalTemplateSet = new HashSet<>();
        if (!keywordList.isEmpty()) {
            for (String keyword : keywordList) {
                //根据keyword获取运费模板
                List<TbCfExpressTemplate> tbCfExpressTemplateList = templateRepository.getTemplateByKeyword(keyword);
                totalTemplateSet.addAll(tbCfExpressTemplateList);
                //TODO 后续优化
                if (ValidateUtils.isContainChinese(keyword)) {
                    if ( 0 == tbCfExpressTemplateList.size()) {
                        String[] split = keyword.split("");
                        for (int i = 0; i < split.length; i++) {
                            if (i + 1 <= split.length) {
                                List<TbCfExpressTemplate> tbCfExpressTemplateLists = templateRepository.getTemplateByKeyword(split[i] + split[i + 1]);
                                totalTemplateSet.addAll(tbCfExpressTemplateLists);
                            }
                        }
                    }
                }
            }
        }
        //没有的话就设置 其他
        if (totalTemplateSet.isEmpty()) {
            //根据keyword获取运费模板
            List<TbCfExpressTemplate> tbCfExpressTemplateList = templateRepository.getTemplateByKeyword("其他");
            totalTemplateSet.addAll(tbCfExpressTemplateList);
        }
        //取第一个运费
        Iterator<TbCfExpressTemplate> iterator = totalTemplateSet.iterator();
        return iterator.next();
    }


    private Map<String, Object> separateText(String text) throws ExecutionException, InterruptedException, TimeoutException {
        List<Map<String, Object>> futureList = new ArrayList<>();
        Map<String, Object> titleMap = new HashMap<>();
        titleMap.put("text", text);
        WordposHelper.separeteText(futureList, titleMap, text);
        WordposHelper.waitForResult(futureList);
        return titleMap;
    }

}
